/**
 *
 */
package jp.agentec.sinaburocast.action.admin.member;

import java.io.InputStream;
import java.util.List;

import jp.agentec.sinaburocast.action.AbstractAction;
import jp.agentec.sinaburocast.common.SinaburoConstant;
import jp.agentec.sinaburocast.common.annotation.OperationHistoryAno;
import jp.agentec.sinaburocast.common.util.PropertyUtil;
import jp.agentec.sinaburocast.common.util.SinaburoUtil;
import jp.agentec.sinaburocast.common.util.SinaburoViewUtil;
import jp.agentec.sinaburocast.csv.DeliveryGroupInfoCsv;
import jp.agentec.sinaburocast.entity.AdminUser;
import jp.agentec.sinaburocast.entity.DeliveryGroup;
import jp.agentec.sinaburocast.entity.UploadError;
import jp.agentec.sinaburocast.form.admin.member.DeliveryGroupRegistAllForm;
import jp.agentec.sinaburocast.service.DeliveryGroupService;
import jp.agentec.sinaburocast.service.UploadErrorService;
import jp.agentec.sinaburocast.service.csv.CsvService;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;
import org.apache.struts.Globals;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.apache.struts.util.TokenProcessor;
import org.seasar.framework.util.FileUtil;
import org.seasar.framework.util.StringUtil;
import org.seasar.struts.annotation.ActionForm;
import org.seasar.struts.annotation.Execute;

/**
 *
 * 配信グループ一括登録
 * @author kim-hs
 *
 */
public class DeliveryGroupRegistAllAction extends AbstractAction {

	private final Logger logger = Logger.getLogger(getClass());

	@ActionForm
	public DeliveryGroupRegistAllForm deliveryGroupRegistAllForm;

	public CsvService<DeliveryGroupInfoCsv> csvService;

	public UploadErrorService uploadErrorService;

	public DeliveryGroupService deliveryGroupService;

	//ファイルタイプID
	private final String type="2";

	/**
	 * 初期表示
	 */
	@Execute(validator = false)
	public String index() {
		deliveryGroupRegistAllForm.uploadError = uploadErrorService.findByType(type);
    	// トークン発行
		TokenProcessor.getInstance().saveToken(request);

		return "/admin/member/deliveryGroupInputAll.html";
	}

	/**
	 * 配信グループ一括登録
	 * @return
	 * @throws Exception
	 */
	@Execute(validator=true,input = "/admin/member/deliveryGroupRegistAll")
	@OperationHistoryAno(operationTypeId=SinaburoConstant.OperationType.UPLOAD,target="配信グループ一括登録",detail="アップロード",screenId=SinaburoConstant.screenId.CDG0004)
	public String groupMemberUpload() throws Exception {

    	// トークン確認
   		if (!TokenProcessor.getInstance().isTokenValid(request, true)) {

   			return "/error/doubleAction.html";
    	}

		int result = 0;
		String csvData = null;
		InputStream in = null;

		AdminUser adminUser = getAdmin();

		if(!deliveryGroupRegistAllForm.file.getFileName().toUpperCase().endsWith(".TSV")){
			ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
			if (messages == null) {
				messages = new ActionMessages();
			}
			messages.add("file", new ActionMessage(SinaburoViewUtil.getMessage("errors.E040")));
			request.setAttribute(Globals.ERROR_KEY, messages);

			return "/admin/member/deliveryGroupRegistAll/";
		}

		try {
			in = deliveryGroupRegistAllForm.file.getInputStream();
            //バイナリ形式チェック
			byte[] b = new byte[1];
            while (in.read(b, 0, 1) > 0) {
                if (b[0] == 0) {
    				ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
    				if (messages == null) {
    					messages = new ActionMessages();
    				}
    				messages.add("file", new ActionMessage(SinaburoViewUtil.getMessage("errors.E042")));
    				request.setAttribute(Globals.ERROR_KEY, messages);

        			//addError("errors.E042");
        			return "/admin/member/deliveryGroupRegistAll/";
                }
            }

		} finally {
			IOUtils.closeQuietly(in);
		}

		try {
			in = deliveryGroupRegistAllForm.file.getInputStream();
			csvData = IOUtils.toString(in, "Shift_JIS");

		} finally {
			IOUtils.closeQuietly(in);
		}

		StringBuffer isOk   = new StringBuffer("OK");
		//List<DeliveryGroupInfoCsv> groupCsvList = csvService.parseAll(csvData, DeliveryGroupInfoCsv.class,isOk);
		List<DeliveryGroupInfoCsv> groupCsvList = null;

		try {
			groupCsvList = csvService.parseAll(csvData, DeliveryGroupInfoCsv.class,isOk);
		} catch (org.seasar.s2csv.csv.exception.CSVFormatException e) {
			//addError("errors.E039");
			ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
			if (messages == null) {
				messages = new ActionMessages();
			}
			messages.add("file", new ActionMessage(SinaburoViewUtil.getMessage("errors.E039")));
			request.setAttribute(Globals.ERROR_KEY, messages);

			return "/admin/member/deliveryGroupRegistAll/";
		} catch (NumberFormatException e) {
			//addError("errors.E039");
			ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
			if (messages == null) {
				messages = new ActionMessages();
			}
			messages.add("file", new ActionMessage(SinaburoViewUtil.getMessage("errors.E039")));
			request.setAttribute(Globals.ERROR_KEY, messages);
			return "/admin/member/deliveryGroupRegistAll/";
		}



		if (CollectionUtils.isEmpty(groupCsvList)) {
			//addError(SinaburoViewUtil.getMessage("errors.E038"));
			//addError("errors.E039");
			ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
			if (messages == null) {
				messages = new ActionMessages();
			}
			messages.add("file", new ActionMessage(SinaburoViewUtil.getMessage("errors.E038")));
			request.setAttribute(Globals.ERROR_KEY, messages);
			return "/admin/member/deliveryGroupRegistAll/";
		}

		/**結果登録*/
		UploadError uploadError = new UploadError();
		uploadError.filePath = PropertyUtil.getProperty(SinaburoConstant.filePath.UPLOAD_ERROR_FILE_PATH);
		uploadError.filePath += "group_upload_error.tsv";
		uploadError.type=type;//市民団体一括アップロード
		uploadError.status=  StringUtil.equals(isOk.toString(), "NG") ? "NG" : "OK";
		uploadErrorService.deleteInsertByType(uploadError,getAdmin());
		deliveryGroupRegistAllForm.uploadError = uploadErrorService.findByType(type);
		/**結果ファイル作成*/
		//20170508 SJIS文字化け対応
		String outStr = csvService.getContents(groupCsvList, DeliveryGroupInfoCsv.class);
		outStr = SinaburoUtil.toSJIS(outStr);
		FileUtil.write(uploadError.filePath,outStr.getBytes("Windows-31J"));
		//FileUtil.write(uploadError.filePath,csvService.getContents(groupCsvList, DeliveryGroupInfoCsv.class).getBytes("Shift_JIS"));

		/**データチェックエラー(1件でも)時には「市民活動団体一括登録」処理を行わない*/
		if(StringUtil.equals(isOk.toString(), "NG")){
			//addError(SinaburoViewUtil.getMessage("errors.E036"));
			ActionMessages messages = (ActionMessages)request.getAttribute(Globals.ERROR_KEY);
			if (messages == null) {
				messages = new ActionMessages();
			}
			messages.add("file", new ActionMessage(SinaburoViewUtil.getMessage("errors.E036")));
			request.setAttribute(Globals.ERROR_KEY, messages);
	    	// トークン発行
			TokenProcessor.getInstance().saveToken(request);
			return "/admin/member/deliveryGroupInputAll.html";
		}


		DeliveryGroup deliveryGroup = new DeliveryGroup();

		deliveryGroup.deliveryGroupName = deliveryGroupRegistAllForm.groupName;
		deliveryGroup.memo = deliveryGroupRegistAllForm.groupMemo;
		deliveryGroup.delFlg = 0;

		/**一括登録*/
		result = deliveryGroupService.insertDeliveryGroupUpload(deliveryGroup,groupCsvList, adminUser.loginId);
		logger.info("一括更新件数："+result);

		return "/admin/member/deliveryGroupInputAllEnd.html";

	}

	/**
	 * 最後に登録したファイルの結果ファイルをダウンロード
	 * @return
	 * @throws Exception
	 */
	@Execute(validator = false)
	@OperationHistoryAno(operationTypeId=SinaburoConstant.OperationType.DOWNLOAD,target="配信グループ一括登録エラー",detail="エラーダウンロード",screenId=SinaburoConstant.screenId.CDG0004)
	public String resultFileDownLoad()throws Exception {

		 UploadError uploadError = uploadErrorService.findByType(type);
		 jp.agentec.sinaburocast.common.io.FileUtil.fileDownload(uploadError.filePath, "Shift_JIS");

		 deliveryGroupRegistAllForm.uploadError = uploadErrorService.findByType(type);

		return null;
	}

}
